
/**
 * Represents an abstract Scheme-like list, an  immutable linear recursive
 * structure of data. Has abstract methods to provide its internal data and
 * structure to the client. Since an AList has an internal structure that is
 * isomorphic to itself, it's best to implement it using the composite pattern.
 *
 * @author Dung X. Nguyen
 * @version 1.0
 * @since 09/10/00
 * @Custom Copyright 2000 -All rights reserved
 */
public abstract class AList {
    /**
     * Returns the first element in this <code>AList</code>, if any.
     * The behavior for the empty list is undefined.
     */
    public abstract Object getFirst();

    /**
     * Returns the tail ("rest") of this <code>AList</code>, if any.
     * The behavior for the empty list is undefined.
     */
    public abstract AList getRest();

    /**
     * Returns the number of elements in this <code>AList</code>.
     */
    public abstract int length();

    /**
     * A tail recursive equivalent of the method length().
     * Uses the "helper" method helpGetLength.
     */
    public abstract int getLength();

    /**
     * Computes the length of this <code>AList</code> given the accumulated
     * length of the part of the list that precedes it.
     * @param acc >=0, the accumulated length.
     */
    protected abstract int helpGetLength(int acc);
}

